# -*- coding: utf-8 -*-


class Unidade(object):
    """ """
    def __init__(self, base):
        assert isinstance(base, list), 'Only lists are accepted'
        self.base = map(unicode, base)  # [u'DPF', u'CGTI', u'DINF', u'SDS']

    def _reverse_str(self, unity):
        """
        Recebe uma unidade no formato string a inverte.
        Ex.:
        >>> self.reverse_str("SDS/DINF/CGTI/DPF")
        >>> "DPF/CGTI/DINF/SDS"
        """
        result = reversed(unity.split("/"))
        return "/".join(result)

    def normalize(self):
        """
        Retorna a unidade instanciada em self.base em string no formato invertido.
        Ex.:
        >>> self.base
        >>> [u'DPF', u'CGTI', u'DINF', u'SDS']
        >>> self.normalize()
        >>> "SDS/DINF/CGTI/DPF"
        """
        return self._reverse_str("/".join(self.base))

    def match(self, unities):
        """
        Checa se todas as unidades em self.base casam com as
        unidades passadas em 'unities' em ordem (inversa) e valor.
        Ex.:
        >>> self.base
        >>> [u'DPF', u'CGTI', u'DINF', u'SDS']
        >>> self.match([u"SDS", u"DINF", u"GGTI", u"DPF"])
        >>> True
        >>> self.match([u"DINF", u"SDS", u"GGTI", u"DPF"])
        >>> False
        >>> self.match([u"DINF", u"GGTI", u"DPF"])
        >>> True
        """
        check = [x for x in reversed(unities)]
        result = map(lambda x: x[0] == x[1], zip(self.base, check))
        return all(result)

    def reverse(self, unities):
        """
        Recebe uma lista de unidades e as retorna no formato reverso.
        Ex.:
        >>> self.reverse([u'SST/DINF/CGTI/DPF', u'SDS/DINF/CGTI/DPF'])
        >>> [u'DPF/CGTI/DINF/SST', u'DPF/CGTI/DINF/SDS']
        """
        nl = map(lambda x: self._reverse_str(x), unities)
        result = [x for x in nl]
        return result

    def sort(self, unities):
        """
        Recebe uma lista de unidades e as ordena em ordem alfabética
        >>> unities = [u'SDS/DINF/CGTI/DPF', u'DINF/CGTI/DPF', u'GAB/CGTI/DPF', u'CGTI/DPF']
        >>> self.sort(unities)
        >>> [u'CGTI/DPF', u'DINF/CGTI/DPF', u'SDS/DINF/CGTI/DPF', u'GAB/CGTI/DPF']
        """
        unities_l = self.reverse(unities)  # Reverte as unidades
        s_result = sorted(unities_l)  # Ordena em ordem alfabética
        result = self.reverse(s_result)  # Resultado invertido
        return result
