# -*- coding: utf-8 -*-

import json

from Products.statusmessages.interfaces import IStatusMessage
from plone import api
from five import grok
from zope.interface import Interface
# from collections import OrderedDict

from pf.listatelefonica.pessoa import IPessoa
from pf.listatelefonica.browser.unidades import Unidade


grok.templatedir('templates')


class ListaTelefonicaView(grok.View):
    grok.context(Interface)
    grok.require('zope2.View')
    grok.name('listatelefonica_view')

    def _checkRegistry(self, username):
        """ Verifica se o usuario ja possui um cadastro na lista telefonica e retorna este """
        # Search for data in portal_catalog tool
        portal_url = self.context.absolute_url()
        catalog = api.portal.get_tool(name='portal_catalog')
        brains = catalog(portal_type="pf.listatelefonica.pessoa",
                         id=username,
                         Creator=username)
        if brains:  # if user data exists, returns url to edit
            user_data = brains[0].getObject().absolute_url()
            target = "{0}/edit".format(user_data)
            return target

        target = "{0}/++add++pf.listatelefonica.pessoa".format(portal_url)
        return target

    def registry(self):
        """ Obtem o usuario logado no portal, verifica se o mesmo ja possui
        cadastro e fornece a url para login/adicionar/editar """

        # Check if actual user is logged on portal
        if api.user.is_anonymous():
            # messages = IStatusMessage(self.request)
            # messages.add(u'Para editar seus dados você precisa antes se identificar.', type=u'info')
            # messages.add(u'entre com seu login e senha nos campos abaixo para continuar.', type=u'info')
            portal_url = self.context.absolute_url()
            login_url = "{0}/login_form?came_from=after_login".format(portal_url)
            return login_url

        # Get username
        current = api.user.get_current()
        username = current.getUserName()

        target = self._checkRegistry(username)

        return target

    def get_description(self):
        """ Verifica se existe um objeto do tipo 'Página' com o mesmo ID do diretório que se encontra
        Se houver, retorna o corpo do texto deste objeto como descrição da Home. """
        folder_path = '/'.join(self.context.getPhysicalPath())
        catalog = api.portal.get_tool(name='portal_catalog')
        brains = catalog(portal_type="Document",
                         id=self.context.id,
                         review_state='published',
                         path={'query': folder_path, 'depth': 1},
                         sort_limit=1)[:1]
        if brains:
            obj = brains[0].getObject()
            return obj.text.output

        return self.context.Description()


class PessoaMixin(object):

    def is_owner(self):
        # Get username
        current = api.user.get_current()
        username = current.getUserName()

        # Check if logged user is object's creator
        owner = self.context.Creator()
        if username == owner:
            return True
        return False

    def was_deleted(self):
        if self.is_owner() and self.context.excluir_dados:
            return True
        return False


class PessoaView(PessoaMixin, grok.View):
    grok.context(IPessoa)
    grok.require('zope2.View')
    grok.name('view')

    def get_ramais(self, context):
        ramal1 = context.ramal1 or ''
        ramal2 = context.ramal2

        if not ramal2:
            return "<span>{0}</span>".format(ramal1)

        return "<span>{0}</span> <br /> <span>{1}</span>".format(ramal1, ramal2)


class SelfDeleteData(PessoaMixin, grok.View):
    """ """
    grok.context(IPessoa)
    grok.require('zope2.View')
    grok.name('lista_delete_data')

    def __call__(self):
        if not self.was_deleted() or not self.is_owner():
            return self.redirect(self.context.absolute_url())
        self.delete()
        return super(SelfDeleteData, self).__call__()

    def delete(self):
        if 'cancel' in self.request.form and self.request.form['cancel'] == 'yes':
            import transaction
            self.context.excluir_dados = False
            transaction.commit()
            return self.redirect(self.context.absolute_url())
        else:
            api.content.delete(self.context)
            messages = IStatusMessage(self.request)
            messages.add(u'Seus dados foram excluídos com sucesso!', type=u'info')
            portal_url = api.portal.get().absolute_url()
            url_return = "{0}/lista-telefonica".format(portal_url)
            return self.redirect(url_return)


class UnidadesView(grok.View):
    grok.name('unidade')
    grok.require('zope2.View')
    grok.context(Interface)

    def __call__(self):
        query = self.request.get('q', '').upper()
        if query.endswith('/'):
            query = query[:-1]
        self.unidade = Unidade(query.split('/'))
        return super(UnidadesView, self).__call__()

    def brains(self):
        pcatalog = api.portal.get_tool(name='portal_catalog')
        path = '/'.join(api.portal.get().getPhysicalPath()) + '/lista-telefonica'
        search = self.unidade.normalize().split('/')
        query = {
            'portal_type': ['pf.listatelefonica.pessoa', ],
            'path': {'query': path, 'depth': 99},
            'unidade': search,
            'sort_on': 'sortable_title',
            'sort_order': 'ascending',
        }
        results = pcatalog(**query)
        return results

    def get_ramais(self, context):
        ramal1 = context.ramal1 or ''
        ramal2 = context.ramal2

        if not ramal2:
            return "<span>{0}</span>".format(ramal1)

        return "<span>{0}</span> <br /> <span>{1}</span>".format(ramal1, ramal2)

    def person2dict(self, obj, brain=None):
        """ """
        people = [{
            'nome': obj.nome,
            'cargo': obj.cargo,
            'funcao': obj.funcao,
            'email': "{0}@dpf.gov.br".format(obj.Title()),
            'ramal': self.get_ramais(obj),
            'celular': obj.celular,
            'voip': obj.voip,
            'url': brain.getURL()
        }]
        return people

    def get_results(self):
        """
        Retorna uma lista com as unidades ordenadas e um dicionário Unidade/Pessoas.
        Ex.:
        [
            ["CGTI/DPF", "CGTI/DPF/DINF", "CGTI/DPF/DINF/SDS"],
            {
                "CGTI/DPF": [<Pessoa 1>, <Pessoa 2> ],
                "CGTI/DPF/DINF/SDS": [<Pessoa 3>, <Pessoa 4>],
                "CGTI/DPF/DINF": [<Pessoa 5>, <Pessoa 6>]
            }
        ]
        """
        results = {}
        unity_list = []

        brains = self.brains()
        if not brains:
            return []

        for brain in brains:
            obj = brain.getObject()

            # Remove espaços em branco no nome de cada unidade
            un = map(lambda x: x.strip(), obj.unidade.split('/'))

            # Checa se a unidade cadastrada pelo usuário casa com a unidade buscada na URL
            match = self.unidade.match(un)
            if not match:
                continue

            people = self.person2dict(obj, brain)

            unity = '/'.join(un)

            # verifica se a unidade já é uma chave do dicionário results
            if not (unity in results):
                results[unity] = []

            # verifica se a unidade já está na lista geral das unidades encontradas
            if not (unity in unity_list):
                unity_list.append(unity)

            results[unity] += people

        # Usado para tal:condition no template. Resultado default retorna: [[],{}]
        if not unity_list:
            return []

        # Ordena as unidades de acordo com a árvore
        unidades = self.unidade.sort(unity_list)

        return [unidades, results]

    def update(self):
        self.persons = self.get_results()

    def registry(self):
        """ """
        portal_url = self.context.absolute_url()
        login_url = "{0}/login_form?came_from=after_login".format(portal_url)
        return login_url


class UnidadesIframeView(UnidadesView):
    grok.name('unidade_iframe')


class UnidadesJsonView(UnidadesView):
    grok.name('unidade_json')

    def get_ramais(self, context):
        ramal1 = context.ramal1 or ''
        ramal2 = context.ramal2

        if not ramal2:
            return "{0}".format(ramal1)

        return "{0} / {1}".format(ramal1, ramal2)

    def get_results(self):
        results = super(UnidadesJsonView, self).get_results()
        if results:
            results.pop(0)
        return results

    def update(self):
        self.request.response.setHeader("Content-type", "application/json")
        self.persons = json.dumps(self.get_results())
