# -*- coding: utf-8 -*-

import re

from plone import api
from five import grok

from zope.interface import Interface

from Products import AdvancedQuery

from pf.listatelefonica.browser.views import ListaTelefonicaView


grok.templatedir('templates')


class ListaCadastroView(grok.View):
    grok.context(Interface)
    grok.require('zope2.View')
    grok.name('listacadastro_view')


class ListaTelefonicaBatch(grok.View):
    grok.context(Interface)
    grok.require('zope2.View')
    grok.name('listatelefonica_batch')


class AfterLoginHandler(grok.View):
    """ classe responsável por verificar se o usuário possui cadastro após o login
    quando o usuário clica em 'editar dados' e o direciona para o local adequado """

    #TODO: Implementar regra de conteúdo para adicionar todas os tipos 'Pessoa' dentro da Lista Telefônica.

    grok.context(Interface)
    grok.require('zope2.View')
    grok.name('after_login')

    def _checkRegistry(self, username):
        """ Verifica se o usuario ja possui um cadastro na lista telefonica e retorna este """
        # Search for data in portal_catalog tool
        portal_url = self.context.absolute_url()
        catalog = api.portal.get_tool(name='portal_catalog')
        brains = catalog(portal_type="pf.listatelefonica.pessoa",
                         id=username,
                         Creator=username)
        if brains:  # if user data exists, returns url to edit
            user_data = brains[0].getObject().absolute_url()
            target = "{0}/edit".format(user_data)
            return target

        target = "{0}/++add++pf.listatelefonica.pessoa".format(portal_url)
        return target

    def render(self):
        portal_url = self.context.absolute_url()

        if api.user.is_anonymous():
            self.redirect(portal_url)

        # Get username
        current = api.user.get_current()
        username = current.getUserName()

        target = self._checkRegistry(username)
        self.redirect(target)


class ListaTelefonicaBuscaView(ListaTelefonicaView):
    grok.name('listatelefonica_busca_view')

    results = []

    def _parse_query_string(self, string):
        """ Remove strings vazias e caracteres não aceitos na busca """
        pattern = re.compile("[^\w]|_")
        result = pattern.sub('', string)
        return result

    def _reverse_str(self, unity):
        """
        Recebe uma unidade no formato string
        Ex.:
        >>> self.reverse_str("SDS/DINF/CGTI/DPF") 
        >>> "DPF/CGTI/DINF/SDS"
        """
        result = reversed(unity.split("/"))
        return "/".join(result)


    def result2dict(self, brains):
        results = []
        for brain in brains:
            obj = brain.getObject()
            results.append({
                'nome': obj.nome,
                'email': "{0}@dpf.gov.br".format(obj.title),
                'url': brain.getURL,
                'url_unidade': "@@unidade?q={0}".format(self._reverse_str(obj.unidade)), 
                'unidade': obj.unidade,
                'ramais': self.get_ramais(obj),
                'celular': obj.celular,
                'voip': obj.voip,
                'cargo': obj.cargo,
                'funcao': obj.funcao
            })
        return results

    def get_ramais(self, item):
        ramal1 = item.ramal1 or ''
        ramal2 = item.ramal2

        if not ramal2:
            return "<span>{0}</span>".format(ramal1)

        return "<span>{0}</span> <br /> <span>{1}</span>".format(ramal1, ramal2)

    def advanced_search(self, form, fields=[]):
        """ """
        if not fields:
            return

        path = '/'.join(self.context.getPhysicalPath())
        query = AdvancedQuery.Eq('portal_type', 'pf.listatelefonica.pessoa') & \
            AdvancedQuery.Eq('path', path)

        for field in fields:
            # val = self._parse_query_string(form.get(field))
            val = form.get(field)
            if val:
                search_ = val.split(' ')
                search = filter(self._parse_query_string, search_)
                search = ' '.join(search)
                if field == 'telefone':
                    telefone = (AdvancedQuery.Eq('ramal1', search) |
                                AdvancedQuery.Eq('ramal2', search) |
                                AdvancedQuery.Eq('celular', search))
                    query = query & telefone
                    continue
                query = query & AdvancedQuery.Eq(field, search)
        return query

    def update(self):
        """ """
        form = self.request.form
        pcatalog = api.portal.get_tool(name='portal_catalog')

        if 'submited' in form.keys():

            # Simple search
            title = form.get('title', '')
            if title:
                search_ = title.split(' ')
                search = filter(self._parse_query_string, search_)
                path = '/'.join(self.context.getPhysicalPath())
                query = {
                    'portal_type': ['pf.listatelefonica.pessoa', ],
                    'path': {'query': path, 'depth': 99},
                    'SearchableText': search,
                    'sort_on': 'sortable_title',
                    'sort_order': 'ascending',
                }
                results = pcatalog(**query)

            # Advanced search
            else:
                fields = ['telefone', 'nome', 'voip',
                          'unidade', 'cargo', 'funcao']
                query = self.advanced_search(form, fields)
                sort_order = (('sortable_title', 'ascending'), )
                results = pcatalog.evalAdvancedQuery(query, sort_order)

            self.results = self.result2dict(results)

