# -*- coding: utf-8 -*-

from plone import api
from zope import schema
from plone.dexterity.content import Item
from plone.supermodel import model
from plone.directives import form
from five import grok

from zope.lifecycleevent.interfaces import IObjectCreatedEvent, IObjectModifiedEvent
from collective import dexteritytextindexer


class IPessoa(model.Schema):
    """
    Pessoa - dados de uma pessoa para lista telefonica
    """
    # Email is readonly only for display at new register.
    # Its don't used in others code's parts
    form.mode(email='display')
    email = schema.TextLine(
        title=u"Email",
        description=u"",
        required=False,
        default=u"",
    )

    dexteritytextindexer.searchable('title')
    form.mode(title='hidden')
    title = schema.TextLine(
        title=u"User ID",
        description=u'ID do usuario - Obtido através do LDAP',
        required=False
    )

    dexteritytextindexer.searchable('nome')
    nome = schema.TextLine(
        title=u" Nome",
        description=u"Informe seu nome",
        required=True
    )

    dexteritytextindexer.searchable('ramal1')
    ramal1 = schema.TextLine(
        title=u"Telefone 1",
        description=u"Informe seu telefone de contato. Ex.: (22) 2222-2222",
        required=True
    )

    dexteritytextindexer.searchable('ramal2')
    ramal2 = schema.TextLine(
        title=u"Telefone 2",
        description=u"Informe outro número, se houver. Ex.: (33) 3333-3333",
        required=False
    )

    dexteritytextindexer.searchable('celular')
    celular = schema.TextLine(
        title=u"Celular",
        description=u"Informe seu telefone celular Ex.: (44) 4444-44444 ",
        required=False
    )

    dexteritytextindexer.searchable('voip')
    voip = schema.TextLine(
        title=u"VOIP",
        description=u"Informe o número VOIP. Ex.: 4 2222 2222",
        required=False
    )

    dexteritytextindexer.searchable('unidade')
    unidade = schema.TextLine(
        title=u"Unidade",
        description=u"Informe a unidade. Ex.: SDS/DINF/CGTI/DPF",
        required=True
    )

    dexteritytextindexer.searchable('cargo')
    cargo = schema.TextLine(
        title=u"Cargo",
        description=u"Informe seu cargo",
        required=True
    )

    dexteritytextindexer.searchable('funcao')
    funcao = schema.TextLine(
        title=u"Função",
        description=u"Informe sua função",
        required=False
    )

    excluir_dados = schema.Bool(
        title=u"Apagar meus dados",
        description=u"Selecione essa opção apenas se tiver absoluta certeza de que deseja apagar todos os seus dados da Lista Telefônica.",
        required=False,
        default=False,
    )

    model.fieldset(
        'privacidade',
        label=u"Privacidade",
        fields=['excluir_dados', ]
    )


@form.default_value(field=IPessoa['email'])
def default_email(data):
    if api.user.is_anonymous():
        return u""
    current = api.user.get_current()
    username = current.getUserName()
    return "{0}@dpf.gov.br".format(username)


class Pessoa(Item):
    grok.implements(IPessoa)

    def get_user_id(self):
        # Get username
        if api.user.is_anonymous():
            return 'isAnon'

        current = api.user.get_current()
        username = current.getUserName()
        return username


def adjust_display_text(object):
    object.cargo = object.cargo.upper()
    if object.funcao:
        object.funcao = object.funcao.title()
    object.unidade = object.unidade.upper()
    return


@grok.subscribe(IPessoa, IObjectCreatedEvent)
def set_user_id(object, event):
    """ O ID do objeto é gerado através do campo 'title' preenchido
    automaticamente com o id obtido do LDAP """
    object.title = object.get_user_id()
    object.email = "{0}@dpf.gov.br".format(object.get_user_id())
    object.exclude_from_nav = True
    adjust_display_text(object)
    return


@grok.subscribe(IPessoa, IObjectModifiedEvent)
def set_display_text(object, event):
    adjust_display_text(object)
    return
