# -*- coding: utf-8 -*-

from Products.CMFCore.utils import getToolByName

from plone import api
from plone.app.textfield.value import RichTextValue


class Empty(object):
    """ """


def install_index(context):
    portal = context.getSite()

    # ADIÇÃO DOS INTEX DE CATALOG
    catalog = getToolByName(portal, 'portal_catalog')
    indexes = catalog.indexes()
    # Specify the indexes you want, with ('index_name', 'index_type')
    wanted = (
        ('nome', 'ZCTextIndex'),
        ('ramal1', 'ZCTextIndex'),
        ('ramal2', 'ZCTextIndex'),
        ('celular', 'ZCTextIndex'),
        ('voip', 'ZCTextIndex'),
        ('unidade', 'ZCTextIndex'),
        ('cargo', 'ZCTextIndex'),
        ('funcao', 'ZCTextIndex'),
    )
    indexables = []
    for name, meta_type in wanted:
        if name not in indexes:
            # Required for ZCTextIndex indexes
            title_extras = Empty()
            title_extras.index_type = 'Okapi BM25 Rank'
            title_extras.lexicon_id = 'plone_lexicon'

            # Add indexes on catalog
            catalog.addIndex(name, meta_type, title_extras)
            indexables.append(name)

    if len(indexables) > 0:
        catalog.manage_reindexIndex(ids=indexables)


def container_has_content(container, content_id):
    try:
        container[content_id]
        return True
    except KeyError:
        return False


def get_or_create_content(container, type, id, title, **kwargs):
    try:
        content = container[id]
    except KeyError:
        content = api.content.create(
            container=container,
            type=type,
            id=id,
            title=title,
            **kwargs)
    return content


def create_initial_content(context):
    portal = api.portal.get()

    # Lista Telefônica page
    lista = get_or_create_content(
        container=portal,
        type='Folder',
        id='lista-telefonica',
        title='Lista Telefônica',
        description='Esse serviço está temporariamente indisponível'
    )
    lista.setLayout('listatelefonica_view')
    lista.manage_setLocalRoles("AuthenticatedUsers", ["Contributor", ])

    # Lista Telefônica description page
    if not container_has_content(lista, 'lista-telefonica'):
        description_text = u"""\
        <p>Descrição do serviço de Lista Telefônica </p>
        """
        obj_desc = get_or_create_content(
            container=lista,
            type='Document',
            id='lista-telefonica',
            title='Descrição do serviço',
            text=RichTextValue(
                description_text,
                'text/html',
                'text/plain'
            )
        )
        if api.content.get_state(obj=obj_desc) != 'published':
            api.content.transition(obj=obj_desc, transition='publish')

        if api.content.get_state(obj=lista) != 'published':
            api.content.transition(obj=lista, transition='publish')

    # Folder Serviços do Portal: used to include links to portal services
    sp = get_or_create_content(
        container=portal,
        type='Folder',
        id='servicos-do-portal',
        title='Serviços do Portal',
        exclude_from_nav=True,
    )
    if api.content.get_state(obj=sp) != 'published':
        api.content.transition(obj=sp, transition='publish')

    get_or_create_content(
        container=sp,
        type='Link',
        id='lista-telefonica',
        title='Lista Telefônica',
        remoteUrl='${portal_url}/lista-telefonica'
    )
